
import java.util.Vector;

class Vertice {

    int id;

    public Vertice(int num) {
        id = num;
    }

    int getId(){
        return id;
    }

    public String toString() {
        return(""+id);
    }
}
public class Grafo {
    int MAX_NUM_VERTICES;

    int num_arestas;

    double[][] pesos; // matriz de pesos

    // operacoes sero adicionadas a partir daqui
    public Grafo(int max_num_vertices) {
      // atribui numero maximo de vertices
      MAX_NUM_VERTICES = max_num_vertices;

      // cria matriz de pesos
      pesos = new double[MAX_NUM_VERTICES][MAX_NUM_VERTICES];

      for (int i = 0; i < MAX_NUM_VERTICES; i++)
          for (int j = 0; j < MAX_NUM_VERTICES; j++)
              pesos[i][j] = Double.NaN;

      // inicializa variaveis
      num_arestas = 0;
    }

    // as operaes sao descritas a partir daqui
    // 
    public boolean insereAresta(Vertice u, Vertice v, double peso) {

        if (u.getId() < MAX_NUM_VERTICES &&
            v.getId() < MAX_NUM_VERTICES) {
    
            if ( Double.isNaN(pesos[u.getId()][v.getId()]) == true) {
                num_arestas++;
            }

            pesos[u.getId()][v.getId()] = peso;

            return(true);
        } else {
            return(false);
        }
    }

    // imprimir
    public void imprimir() {
        for (int i = 0; i < MAX_NUM_VERTICES; i++) {
            for (int j = 0; j < MAX_NUM_VERTICES; j++) {
                if (Double.isNaN(pesos[i][j]) == false) {
                    System.out.println("peso da aresta ("+i+","+j+") = "+ pesos[i][j]);
                }
            }
        }
    }

    public void imprimirDot() {
        System.out.println("digraph {");
        System.out.println("node [shape=circle];");
        for (int i = 0; i < MAX_NUM_VERTICES; i++) {
            for (int j = 0; j < MAX_NUM_VERTICES; j++) {
                if (Double.isNaN(pesos[i][j]) == false) {
                    System.out.println(i+"->"+j+"[label=\""+ pesos[i][j]+"\"];");
                }
            }
        }

        System.out.println("}");
    }

    public boolean existeAresta(Vertice u, Vertice v) {
        if (Double.isNaN(pesos[u.getId()][v.getId()]) == true) {
            return(false);
        } else {
            return(true);
        }
    }

    
    public Vertice[] listaAdjacentes(Vertice v) {

        int vId = v.getId();
        Vector<Vertice> vetor = new Vector();

        // procura pelos adjacentes
        for (int j = 0; j < pesos[vId].length; j++) {
            if (Double.isNaN(pesos[vId][j]) == false) {
                vetor.add(new Vertice(j));
            }
        }

        Vertice[] adj = (Vertice[]) vetor.toArray(new Vertice[0]);
        return(adj);
    }

    public void removeAresta(Vertice v, Vertice u) {
        int vId = v.getId();
        int uId = u.getId();

        pesos[vId][uId] = Double.NaN;
    }

    Grafo transposto() {

        Grafo gT = new Grafo(MAX_NUM_VERTICES);

        for (int i = 0; i < pesos.length; i++) {
            for (int j = 0; j < pesos[i].length; j++) {
                if (Double.isNaN(pesos[i][j]) == false) 
                  gT.insereAresta(new Vertice(i), new Vertice(j), pesos[i][j]); 
            }
        }

        return(gT);
    }

    public Vertice[] arestaMinima() {

        double min = Double.MAX_VALUE;
        int vId = 0, uId = 0;

        for (int i = 0; i < pesos.length; i++) 
            for (int j = 0; j < pesos[i].length; j++) 
                if (Double.isNaN(pesos[i][j]) == false) 
                    if (pesos[i][j] < min) {
                        uId = i;
                        vId = j;
                    }

        Vertice aresta[] = new Vertice[2];
        aresta[0] = new Vertice(uId);
        aresta[1] = new Vertice(vId);
        return(aresta);
    }

    public static void main(String args[]) {

        Grafo g = new Grafo(10);

        Vertice v0 = new Vertice(0);
        Vertice v1 = new Vertice(1);

        g.insereAresta(v0, v1, 0.5);

        g.imprimir();

        if (g.existeAresta(v0, v1) == true){
            System.out.println("aresta (v0, v1) existe");
        } 

        if (g.existeAresta(v1, v0) == false){
            System.out.println("aresta (v1, v0) no existe");
        } 

        Vertice[] adj = g.listaAdjacentes(v0);

        for (Vertice v: adj) {
            System.out.println("Vertice adjacente "+ v);
        }

        /*
        g.removeAresta(v0,v1);

        adj = g.listaAdjacentes(v0);

        if (adj.length < 1) {
            System.out.println("No h adjacentes");
        }

        for (Vertice v: adj) {
            System.out.println("Vertice adjacente "+ v);
        }*/


        
        Grafo gT = g.transposto();
        System.out.println("Imprimindo grafo transposto gT");
        gT.imprimirDot();

        /*
        System.out.println("Imprimindo grafo g");
        g.imprimirDot();*/

        Vertice[] arestaMinima = g.arestaMinima();

        System.out.println("Aresta mnima = ("+arestaMinima[0]+","+arestaMinima[1]+")");
    }
     
}
              
